"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BootstrapUtils = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const Handlebars = require("handlebars");
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const util = require("util");
const Docker = require("dockerode");
const _ = require("lodash");
const figlet_1 = require("figlet");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const MemoryStream = require('memorystream');
// eslint-disable-next-line @typescript-eslint/no-var-requires
const yaml = require('js-yaml');
// eslint-disable-next-line @typescript-eslint/no-var-requires
const exec = util.promisify(require('child_process').exec);
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class BootstrapUtils {
    static deleteFolder(folder) {
        if (fs_1.existsSync(folder)) {
            logger.info(`Deleting folder ${folder}`);
        }
        return BootstrapUtils.deleteFolderRecursive(folder);
    }
    static deleteFolderRecursive(folder) {
        if (fs_1.existsSync(folder)) {
            fs_1.readdirSync(folder).forEach((file) => {
                const curPath = path_1.join(folder, file);
                if (fs_1.lstatSync(curPath).isDirectory()) {
                    // recurse
                    this.deleteFolderRecursive(curPath);
                }
                else {
                    // delete file
                    fs_1.unlinkSync(curPath);
                }
            });
            fs_1.rmdirSync(folder);
        }
    }
    static deleteFile(file) {
        if (fs_1.existsSync(file) && fs_1.lstatSync(file).isFile()) {
            fs_1.unlinkSync(file);
        }
    }
    static showBanner() {
        console.log(figlet_1.textSync('symbol-bootstrap', { horizontalLayout: 'full' }));
    }
    static createDocker() {
        return new Docker({ socketPath: '/var/run/docker.sock' });
        // return new Docker({ host: 'localhost', port: 2375 });
        // return new Docker({ socketPath: 'tcp://0.0.0.0:2375' });
    }
    static async pullImage(docker, image) {
        if (BootstrapUtils.pulledImages.indexOf(image) > -1) {
            return;
        }
        try {
            await docker.getImage(image).inspect();
            logger.info(`Image ${image} found`);
            BootstrapUtils.pulledImages.push(image);
        }
        catch (e) {
            logger.info(`Pulling image ${image}`);
            //TODO improve how to pull images using dockerone!!!
            const { stdout, stderr } = await exec('docker pull ' + image);
            if (stderr) {
                throw new Error(`Cannot pull image ${image}\n ${stderr}`);
            }
            const outputLines = stdout.toString().split('\n');
            logger.info(`Image pulled: ${outputLines[outputLines.length - 2]}`);
            BootstrapUtils.pulledImages.push(image);
        }
    }
    static async runImageUsingExec(image, userId, cmds, binds) {
        const volumes = binds.map((b) => `-v ${b}`).join(' ');
        const userParam = userId ? `-u ${userId}` : '';
        const runCommand = `docker run --rm ${userParam} ${volumes} ${image} ${cmds.map((a) => `"${a}"`).join(' ')}`;
        logger.info(`Running image using Exec: ${image} ${cmds.join(' ')}`);
        const { stdout, stderr } = await exec(runCommand);
        return { stdout, stderr };
    }
    static async runImageUsingApi(image, userId, cmds, binds) {
        const createOptions = { User: userId, HostConfig: { Binds: binds } };
        const startOptions = {};
        const memStream = new MemoryStream();
        let stdout = '';
        memStream.on('data', (data) => {
            stdout += data.toString();
        });
        logger.info(`Running image using API: ${image} ${cmds.join(' ')}`);
        const docker = BootstrapUtils.createDocker();
        await BootstrapUtils.pullImage(docker, image);
        await docker.run(image, cmds, memStream, createOptions, startOptions);
        return stdout;
    }
    static loadPresetData(root, preset, assembly, customPresetFile) {
        const sharedPreset = this.loadYaml(path_1.join(root, 'presets', 'shared.yml'));
        const networkPreset = this.loadYaml(`${root}/presets/${preset}/network.yml`);
        const assemblyPreset = assembly ? this.loadYaml(`${root}/presets/${preset}/assembly-${assembly}.yml`) : {};
        const customPreset = customPresetFile ? this.loadYaml(customPresetFile) : {};
        //Deep merge
        const presetData = _.merge(sharedPreset, networkPreset, assemblyPreset, customPreset, { preset });
        if (!BootstrapUtils.presetInfoLogged) {
            logger.info(`Generating config from preset ${preset}`);
            if (assembly) {
                logger.info(`Assembly preset ${assembly}`);
            }
            if (customPresetFile) {
                logger.info(`Custom preset file ${customPresetFile}`);
            }
        }
        BootstrapUtils.presetInfoLogged = true;
        if (presetData.assemblies && !assembly) {
            throw new Error(`Preset ${preset} requires assembly (-a, --assembly option). Possible values are: ${presetData.assemblies}`);
        }
        return presetData;
    }
    static loadExistingPresetData(target) {
        return this.loadYaml(`${target}/config/preset.yml`);
    }
    static loadExistingAddresses(target) {
        return this.loadYaml(`${target}/config/generated-addresses/addresses.yml`);
    }
    static sleep(ms) {
        // Create a promise that rejects in <ms> milliseconds
        return new Promise((resolve) => {
            setTimeout(() => {
                resolve();
            }, ms);
        });
    }
    static poll(promiseFunction, totalPollingTime, pollIntervalMs) {
        const startTime = new Date().getMilliseconds();
        return promiseFunction().then(async (result) => {
            if (result) {
                return true;
            }
            else {
                const endTime = new Date().getMilliseconds();
                const newPollingTime = Math.max(totalPollingTime - pollIntervalMs - (endTime - startTime), 0);
                if (newPollingTime) {
                    await BootstrapUtils.sleep(pollIntervalMs);
                    return this.poll(promiseFunction, newPollingTime, pollIntervalMs);
                }
                else {
                    return false;
                }
            }
        });
    }
    static toAmount(renderedText) {
        const numberAsString = (renderedText + '').split("'").join('');
        if (!numberAsString.match(/^\d+$/)) {
            throw new Error(`'${renderedText}' is not a valid integer`);
        }
        return (numberAsString.match(/\d{1,3}(?=(\d{3})*$)/g) || [numberAsString]).join("'");
    }
    static toHex(renderedText) {
        const numberAsString = renderedText.split("'").join('').replace(/^(0x)/, '');
        return '0x' + (numberAsString.match(/\w{1,4}(?=(\w{4})*$)/g) || [numberAsString]).join("'");
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    static async generateConfiguration(templateContext, copyFrom, copyTo) {
        // Loop through all the files in the config folder
        await fs_1.promises.mkdir(copyTo, { recursive: true });
        const files = await fs_1.promises.readdir(copyFrom);
        await Promise.all(files.map(async (file) => {
            const fromPath = path_1.join(copyFrom, file);
            const toPath = path_1.join(copyTo, file);
            // Stat the file to see if we have a file or dir
            const stat = await fs_1.promises.stat(fromPath);
            if (stat.isFile()) {
                if (toPath.indexOf('.mustache') > -1) {
                    const destinationFile = toPath.replace('.mustache', '');
                    const template = await fs_1.promises.readFile(fromPath, 'utf8');
                    Handlebars.registerHelper('toAmount', BootstrapUtils.toAmount);
                    Handlebars.registerHelper('toHex', BootstrapUtils.toHex);
                    const compiledTemplate = Handlebars.compile(template);
                    const renderedTemplate = compiledTemplate(Object.assign({}, templateContext));
                    await fs_1.promises.writeFile(destinationFile, renderedTemplate);
                }
                else {
                    await fs_1.promises.copyFile(fromPath, toPath);
                }
            }
            else if (stat.isDirectory()) {
                await fs_1.promises.mkdir(toPath, { recursive: true });
                await this.generateConfiguration(templateContext, fromPath, toPath);
            }
        }));
    }
    static async mkdir(path) {
        await fs_1.promises.mkdir(path, { recursive: true });
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    static async writeYaml(path, object) {
        const yamlString = this.toYaml(object);
        await BootstrapUtils.writeTextFile(path, yamlString);
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    static toYaml(object) {
        return yaml.safeDump(object, { skipInvalid: true, indent: 4, lineWidth: 140 });
    }
    static fromYaml(yamlString) {
        return yaml.safeLoad(yamlString);
    }
    static loadYaml(fileLocation) {
        return this.fromYaml(fs_1.readFileSync(fileLocation, 'utf8'));
    }
    static async writeTextFile(path, text) {
        await fs_1.promises.writeFile(path, text, 'utf8');
    }
    static async resolveDockerUserFromParam(paramUser) {
        if (!paramUser || paramUser.trim() === '') {
            return undefined;
        }
        if (paramUser === BootstrapUtils.CURRENT_USER) {
            return BootstrapUtils.getDockerUserGroup();
        }
        return paramUser;
    }
    static async getDockerUserGroup() {
        const isWin = process.platform === 'win32';
        if (isWin) {
            return '';
        }
        if (BootstrapUtils.dockerUserId !== undefined) {
            return BootstrapUtils.dockerUserId;
        }
        try {
            const { stdout } = await exec('echo $(id -u):$(id -g)');
            logger.info(`User for docker resolved: ${stdout}`);
            const user = stdout.trim();
            BootstrapUtils.dockerUserId = user;
            return user;
        }
        catch (e) {
            logger.info(`User for docker could not be resolved: ${e}`);
            return '';
        }
    }
}
exports.BootstrapUtils = BootstrapUtils;
BootstrapUtils.CURRENT_USER = 'current';
BootstrapUtils.presetInfoLogged = false;
BootstrapUtils.pulledImages = [];
