"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComposeService = void 0;
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const BootstrapUtils_1 = require("./BootstrapUtils");
const path_1 = require("path");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const fs = require('fs');
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class ComposeService {
    constructor(root, params) {
        this.root = root;
        this.params = params;
    }
    async run(passedPresetData) {
        const presetData = passedPresetData !== null && passedPresetData !== void 0 ? passedPresetData : BootstrapUtils_1.BootstrapUtils.loadExistingPresetData(this.params.target);
        const workingDir = process.cwd();
        const target = path_1.join(workingDir, this.params.target);
        const targetDocker = path_1.join(target, `docker`);
        if (this.params.reset) {
            BootstrapUtils_1.BootstrapUtils.deleteFolder(targetDocker);
        }
        const dockerFile = path_1.join(targetDocker, 'docker-compose.yml');
        if (fs.existsSync(dockerFile)) {
            logger.info(dockerFile + ' already exist. Reusing. (run -r to reset)');
            return;
        }
        await BootstrapUtils_1.BootstrapUtils.mkdir(path_1.join(this.params.target, 'state'));
        await BootstrapUtils_1.BootstrapUtils.mkdir(targetDocker);
        await BootstrapUtils_1.BootstrapUtils.generateConfiguration(presetData, path_1.join(this.root, 'config', 'docker'), targetDocker);
        const user = await BootstrapUtils_1.BootstrapUtils.resolveDockerUserFromParam(this.params.user);
        const vol = (hostFolder, imageFolder) => {
            return hostFolder + ':' + imageFolder;
        };
        logger.info(`creating docker-compose.yml from last used profile.`);
        const services = {};
        (presetData.databases || []).forEach((n) => {
            services[n.name] = {
                image: presetData.mongoImage,
                user,
                command: `bash -c "mongod --dbpath=/dbdata --bind_ip=${n.name}"`,
                stop_signal: 'SIGINT',
                ports: n.openPort ? ['27017:27017'] : [],
                volumes: [vol('../data/mongo', '/dbdata:rw')],
            };
            services[n.name + '-init'] = {
                image: presetData.mongoImage,
                user,
                command: 'bash -c "/bin/bash /userconfig/mongors.sh && touch /state/mongo-is-setup"',
                volumes: [vol(`./mongo`, `/userconfig/:ro`), vol('../data/mongo', '/dbdata:rw'), vol('../state', '/state')],
                depends_on: [n.name],
            };
        });
        (presetData.nodes || []).forEach((n) => {
            const nodeService = {
                image: presetData.symbolServerImage,
                user,
                command: `bash -c "/bin/bash /userconfig/runServerRecover.sh  ${n.name} && /bin/bash /userconfig/startServer.sh ${n.name}"`,
                stop_signal: 'SIGINT',
                depends_on: [],
                restart: 'on-failure:2',
                ports: n.openBrokerPort ? ['7900:7900'] : [],
                volumes: [
                    vol(`./dockerfiles/catapult/userconfig/`, `/userconfig`),
                    vol(`./bin/bash`, `/bin-mount`),
                    vol(`../config/${n.name}/resources/`, `/userconfig/resources/`),
                    vol('../data/' + n.name, '/data:rw'),
                    vol('../data/nemesis-data', '/nemesis-data:rw'),
                    vol('../state', '/state'),
                ],
            };
            if (n.databaseHost) {
                nodeService.depends_on.push(n.databaseHost + '-init');
            }
            services[n.name] = nodeService;
            if (n.brokerHost) {
                services[n.brokerHost] = {
                    image: presetData.symbolServerImage,
                    user,
                    command: `bash -c "/bin/bash /userconfig/runServerRecover.sh ${n.brokerHost} && /bin/bash /userconfig/startBroker.sh ${n.brokerHost}"`,
                    stop_signal: 'SIGINT',
                    ports: n.openBrokerPort ? ['7902:7902'] : [],
                    restart: 'on-failure:2',
                    volumes: [
                        vol(`./dockerfiles/catapult/userconfig/`, `/userconfig`),
                        vol(`./bin/bash`, `/bin-mount`),
                        vol(`../config/${n.name}/resources/`, `/userconfig/resources/`),
                        vol('../data/' + n.name, '/data:rw'),
                        vol('../state', '/state'),
                    ],
                };
                nodeService.depends_on.push(n.brokerHost);
            }
        });
        (presetData.gateways || []).forEach((n) => {
            services[n.name] = {
                image: presetData.symbolRestImage,
                user,
                command: 'ash -c "npm start /userconfig/rest.json"',
                stop_signal: 'SIGINT',
                ports: n.openPort ? ['3000:3000'] : [],
                volumes: [
                    vol(`./bin/ash`, `/bin-mount`),
                    vol(`../config/${n.name}/`, `/userconfig/`),
                    vol('../state', '/state'),
                    vol(`../data/${n.apiNodeHost}`, `/logs:rw`),
                    vol(`../config/${n.apiNodeHost}/resources/`, `/usr/local/share/symbol/api-node-config/`),
                ],
                depends_on: [n.databaseHost + '-init'],
                networks: {
                    default: {
                        ipv4_address: '172.20.0.25',
                    },
                },
            };
        });
        const dockerCompose = {
            version: '3',
            networks: {
                default: {
                    ipam: {
                        config: [
                            {
                                subnet: '172.20.0.0/24',
                            },
                        ],
                    },
                },
            },
            services: services,
        };
        await BootstrapUtils_1.BootstrapUtils.writeYaml(dockerFile, dockerCompose);
        logger.info(`docker-compose.yml file created ${dockerFile}`);
    }
}
exports.ComposeService = ComposeService;
ComposeService.defaultParams = {
    target: 'target',
    user: BootstrapUtils_1.BootstrapUtils.CURRENT_USER,
    reset: false,
};
